{% extends "account/base.html" %}
{% comment %}

authenticate.html:  2fa authenticate view template

SPDX-License-Identifier: Apache-2.0

django-allauth-2f2a, a 2fa adapter for django-allauth.

Copyright 2016-2021 Víðir Valberg Guðmundsson and Percipient
Networks, LLC.
Copyright 2021 Jeremy A Gray <gray@flyquackswim.com>.

Licensed under the Apache License, Version 2.0 (the "License"); you
may not use this file except in compliance with the License.  You
may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
implied.  See the License for the specific language governing
permissions and limitations under the License.

{% endcomment %}

{% load i18n %}

{% block content %}
<h1>
  {% trans "Two-Factor Authentication" %}
</h1>

<form method="post">
  {% csrf_token %}
  {{ form.non_field_errors }}
  {{ form.otp_token.label }}:
  {{ form.otp_token }}

  <button type="submit">
    {% trans 'Authenticate' %}
  </button>
</form>
{% endblock %}
